var __accessCheck = (obj, member, msg) => {
  if (!member.has(obj))
    throw TypeError("Cannot " + msg);
};
var __privateGet = (obj, member, getter) => {
  __accessCheck(obj, member, "read from private field");
  return getter ? getter.call(obj) : member.get(obj);
};
var __privateAdd = (obj, member, value) => {
  if (member.has(obj))
    throw TypeError("Cannot add the same private member more than once");
  member instanceof WeakSet ? member.add(obj) : member.set(obj, value);
};
var __privateSet = (obj, member, value, setter) => {
  __accessCheck(obj, member, "write to private field");
  setter ? setter.call(obj, value) : member.set(obj, value);
  return value;
};

// src/application.ts
import { v4 as uuid } from "uuid";
import { create as createContext } from "sketch-context";
var BaseApplication = class {
  constructor(canvas) {
    this.canvas = canvas;
    this._events = /* @__PURE__ */ new Map();
    this._disposers = /* @__PURE__ */ new Set();
    const ctx = createContext({ canvas: this.canvas });
    if (ctx == null) {
      throw new Error("Erroring creating context");
    }
    this.ctx = ctx;
  }
  on(event) {
    const id = uuid();
    if (!this._events.has(event.type)) {
      this._events.set(event.type, /* @__PURE__ */ new Map());
    }
    const map = this._events.get(event.type);
    if (map == null) {
      throw new Error(`Can not get event map for ${event.type}`);
    }
    map.set(id, event.action);
    return () => {
      map.delete(id);
    };
  }
  dispose() {
    for (let fn of this._disposers) {
      fn();
    }
  }
};

// src/mixins.ts
import { setCanvasSize } from "sketch-canvas-size";
function withResize(Base) {
  return class Resize extends Base {
    constructor(...args) {
      super(...args);
      this._events.set("resize", /* @__PURE__ */ new Map());
      const resizeObserver = new ResizeObserver((entries) => {
        const map = this._events.get("resize");
        for (const entry of entries) {
          const cr = entry.contentRect;
          setCanvasSize({
            canvas: this.canvas,
            width: cr.width,
            height: cr.height
          });
          if (map != null) {
            for (let [_, fn] of map) {
              fn();
            }
          }
          if (this._events.has("tick")) {
            const m = this._events.get("tick");
            if (m != null) {
              for (let [_, fn] of m) {
                fn(0);
              }
            }
          }
        }
      });
      const container = this.canvas.parentElement;
      if (container != null) {
        resizeObserver.observe(container);
      }
      this._disposers.add(() => {
        resizeObserver.disconnect();
      });
    }
    on(event) {
      return super.on(event);
    }
  };
}
function withTick(Base) {
  var _last, _tickID, _isRunning, _tick, _a;
  return _a = class extends Base {
    constructor(...args) {
      super(...args);
      __privateAdd(this, _last, void 0);
      __privateAdd(this, _tickID, void 0);
      __privateAdd(this, _isRunning, false);
      __privateAdd(this, _tick, () => {
        if (__privateGet(this, _isRunning) === false) {
          return;
        }
        const map = this._events.get("tick");
        if (map != null) {
          const now = window.performance.now();
          for (let [_, fn] of map) {
            fn(now - __privateGet(this, _last));
          }
          __privateSet(this, _last, now);
        }
        __privateSet(this, _tickID, window.requestAnimationFrame(__privateGet(this, _tick)));
      });
      __privateSet(this, _tickID, null);
      __privateSet(this, _last, window.performance.now());
      this._events.set("tick", /* @__PURE__ */ new Map());
      this._disposers.add(() => {
        if (__privateGet(this, _tickID)) {
          window.cancelAnimationFrame(__privateGet(this, _tickID));
        }
      });
    }
    start() {
      __privateSet(this, _isRunning, true);
      __privateSet(this, _last, window.performance.now());
      __privateGet(this, _tick).call(this);
    }
    stop() {
      __privateSet(this, _isRunning, false);
    }
    on(event) {
      return super.on(event);
    }
  }, _last = new WeakMap(), _tickID = new WeakMap(), _isRunning = new WeakMap(), _tick = new WeakMap(), _a;
}
export {
  BaseApplication,
  withResize,
  withTick
};
